// exr2depth.cpp
//
// This program can convert 4 Time-of-Flight raw frames into amplitude, intensity, phase and depth map
// This code is unsafe as hell. do some sanitizing before productive use.
//
// Author: Stephan Meister <stephan.meister@iwr.uni-heidelberg.de>
// Heidelberg Collaboratory for Image Processing (HCI),
// Interdisciplinary Center for Scientific Computing (IWR),
// University of Heidelberg, Sep 2013
//

#include <iostream>

#undef cimg_use_openexr
#include "CImg.h"
#include <ImfRgbaFile.h>
#include <ImfInputFile.h>
#include <ImfChannelList.h>
#include <ImfArray.h>

using namespace Imf;
using namespace Imath;
using namespace cimg_library;


void printVersion()
{
	std::cout << "exr2depth : convert OpenEXR ToF raw phase frames into intensity, amplitude, depth and phase maps"
		<< "\n\rStephan Meister, HCI, IWR, Uni Heidelberg 2013 \r\n" << std::endl ;
}

//helper function for writing exr files
void writeRgba1 (const char fileName[],
				const Rgba *pixels,
				int width,
				int height)
{
	RgbaOutputFile file (fileName, width, height, WRITE_RGBA);      // 1
	file.setFrameBuffer (pixels, 1, width);                         // 2
	file.writePixels (height);                                      // 3
}

//helper function for reading exr files
// we don't acutally need the g,b and a channels 
// but we will read them none the less
void readRGBA1 (const char fileName[],
				Array2D<float> &rPixels,
				Array2D<float> &gPixels,
				Array2D<float> &bPixels,
				Array2D<float> &aPixels,
				int &width, int &height)
{
	InputFile file (fileName);
	Box2i dw = file.header().dataWindow();
	width  = dw.max.x - dw.min.x + 1;
	height = dw.max.y - dw.min.y + 1;
	rPixels.resizeErase (height, width);
	gPixels.resizeErase (height, width);
	bPixels.resizeErase (height, width);
	aPixels.resizeErase (height, width);

	FrameBuffer frameBuffer;
	try {
		frameBuffer.insert ("R",                                  // name
			Slice (Imf::FLOAT,                          // type 
					(char *) (&rPixels[0][0] -     // base
								dw.min.x -
								dw.min.y * width),
					sizeof (rPixels[0][0]) * 1,    // xStride
					sizeof (rPixels[0][0]) * width,// yStride
					1, 1,                          // x/y sampling
					0.0));                         // fillValue

		frameBuffer.insert ("G",                                  // name
			Slice (Imf::FLOAT,                          // type
					(char *) (&gPixels[0][0] -     // base
								dw.min.x -
								dw.min.y * width),
					sizeof (gPixels[0][0]) * 1,    // xStride
					sizeof (gPixels[0][0]) * width,// yStride
					1, 1,                          // x/y sampling
					0.0));                         // fillValue

		frameBuffer.insert ("B",                                  // name
			Slice (Imf::FLOAT,                         // type
					(char *) (&bPixels[0][0] -     // base
								dw.min.x -
								dw.min.y * width),
					sizeof (bPixels[0][0]) * 1,    // xStride
					sizeof (bPixels[0][0]) * width,// yStride
					1, 1,                          // x/y sampling
					0.0));                     // fillValue
		
		frameBuffer.insert ("A",                                  // name
			Slice (Imf::FLOAT,                          // type
					(char *) (&aPixels[0][0] -     // base
								dw.min.x -
								dw.min.y * width),
					sizeof (aPixels[0][0]) * 1,    // xStride
					sizeof (aPixels[0][0]) * width,// yStride
					1, 1,                          // x/y sampling
					0.0));                         // fillValue
		
		file.setFrameBuffer (frameBuffer);
		file.readPixels (dw.min.y, dw.max.y);
	}
	catch(std::exception& err)
	{	std::string w = err.what() ;	
		return ;
	}
}


int main(int argc, char* argv[])
{
	if(argc < 6)
	{	
		printVersion() ;
		std::cout	<< "This program expects 4 raw frames in the OpenEXR format.\n"
					<< "Only the Red-Channel is used.\nThe files should be in the order according to the ToF phase shift of 0, 90, 180 and 270 degree.\n"
					<< "Call :  <raw frame 0> <raw frame 1> <raw frame 2><raw frame 3> <outputfile> [Frequency (defaults to 20Mhz)] [output format (cimg, raw, txt; defaults to cimg)]\n" << std::endl;
		return EXIT_FAILURE ;
	}

	float frequency = 2e7 ;
	if(argc == 7)
	{	
		frequency = atof(argv[6]) ;
	}

	char prefix[255] ;
	strcpy(prefix,"cimg") ;
	if(argc == 8)
	{
		strcpy(prefix, argv[7]) ;
	}

	CImgList<float> raw(4) ;

	int width = 0 ;
	int height = 0 ;

	//read 4 OpenEXR images.
	//g,b,a channels will be discarded
	for(int i = 0 ; i < 4 ; i++)
	{
		Array2D<float> r,g,b,a ;
		readRGBA1(argv[i+1], r, g, b, a,width, height) ;
		raw(i).assign(r[0], width, height, 1, 1) ;
	}
	
	CImg<float> intensity, amplitude,phase,depth ;

	intensity.assign(raw(0)) ;
	intensity = (raw(0) + raw(1) + raw(2) + raw(3)) / 4.0 ;
	
	amplitude.assign(raw(0)) ;
	amplitude = (raw(0) - raw(2)).get_pow(2.0) 
		+ (raw(3) - raw(1)).get_pow(2.0) ;
	amplitude = amplitude.get_sqrt() * M_PI / 2.0 ;

	phase.assign(raw(0)) ;
	phase = (raw(3) - raw(1)) ;
	CImg<float> p2 = (raw(0)-raw(2)) ;
	
	phase.atan2(p2) ;
	
	depth.assign(raw(0)) ;
	depth = phase ;
	//shift phase to positive range before calculating depth
	cimg_forXYZC(phase,x,y,z,c)
	{
		float v = phase(x,y,z,c) ;
		depth(x,y,z,c) = (v < 0 ? v + (2.0f* M_PI) : v) ;
	}
	depth = depth / (2.0 * M_PI) * 0.5 * (3e8 / frequency) ;

	char filename[255] ;
	sprintf(filename,"%s_int.%s",argv[5],prefix);
	intensity.save(filename) ;
	
	sprintf(filename,"%s_amp.%s",argv[5],prefix);
	amplitude.save(filename) ;

	sprintf(filename,"%s_phase.%s",argv[5],prefix);
	phase.save(filename) ;

	sprintf(filename,"%s_depth.%s",argv[5],prefix);
	depth.save(filename) ;
	
	return EXIT_SUCCESS;
}

