/*  Copyright (C) 2012 Stephan Meister

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file RadialZConversion.h
 *  Declaration of the parameter class RadialZConversion.
 *  \author <a href="mailto:stephan.meister@iwr.uni-heidelberg.de">
 *      Stephan Meister</a>
 *  \date 30.05.2012
 */

#ifndef _RadialZConversion_H_
#define _RadialZConversion_H_

#ifdef _MSC_VER
#ifdef RadialZConversion_EXPORTS
/// Visual C++ specific code
#define RadialZConversion_DECLDIR __declspec(dllexport)
#else
#define RadialZConversion_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define RadialZConversion_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>

/// Calculated depth, Intensity and Amplitude from raw tof data
/** Calculated depth, Intensity and Amplitude from raw tof data
 */
template <typename T>
class RadialZConversion_DECLDIR RadialZConversion :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	RadialZConversion(const std::string& name = "");

	/// zDepthIn
	InputSlot< cimg_library::CImgList<T> > zDepthIn;
	/// radialDepthIn
	InputSlot< cimg_library::CImgList<T> > radialDepthIn;

	/// camera matrix (optional)
	InputSlot< cimg_library::CImgList<T> > camMatrix;

	/// zDepthOut
	OutputSlot< cimg_library::CImgList<T> > zDepthOut;
	/// radialDepthOut
	OutputSlot< cimg_library::CImgList<T> > radialDepthOut;

	OutputSlot< cimg_library::CImgList<T> > xOut ;
	OutputSlot< cimg_library::CImgList<T> > yOut ;

	/// Update object.
	virtual void execute();
};

#endif // _RadialZConversion_H_
