/*  Copyright (C) 2012 Stephan Meister

	This file is part of Charon.

	Charon is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Charon is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Charon.  If not, see <http://www.gnu.org/licenses/>.
*/
/** \file TofSim.h
 *  Declaration of the parameter class TofSim.
 *  \author <a href="mailto:stephan.meister@iwr.uni-heidelberg.de">
 *      Stephan Meister</a>
 *  \date 30.05.2012
 */

#ifndef _TOFSIM_H_
#define _TOFSIM_H_

#ifdef _MSC_VER
#ifdef tofsim_EXPORTS
/// Visual C++ specific code
#define tofsim_DECLDIR __declspec(dllexport)
#else
#define tofsim_DECLDIR __declspec(dllimport)
#endif /*Export or import*/
#else
/// Not needed without MSVC
#define tofsim_DECLDIR
#endif

#include <charon-core/ParameteredObject.h>
#include <charon-utils/CImg.h>

/// Calculated depth, Intensity and Amplitude from raw tof data
/** Calculated depth, Intensity and Amplitude from raw tof data
 */
template <typename T>
class tofsim_DECLDIR TofSim :
		public TemplatedParameteredObject<T> {
public:
	/// default constructor
	/// \param name          Instance name
	TofSim(const std::string& name = "");

	/// Input Image with phase shift 0
	InputSlot< cimg_library::CImgList<T> > phase0;

	/// Intensity
	OutputSlot< cimg_library::CImgList<T> > intensity;
	/// Amplitude
	OutputSlot< cimg_library::CImgList<T> > amp;
	/// Phase
	OutputSlot< cimg_library::CImgList<T> > phase ;
	/// Depth
	OutputSlot< cimg_library::CImgList<T> > depth;

	/// tof modulation frequency
	Parameter< double > frequency;

	Parameter< double > ambOffset ;

	Parameter < double > cubicCoef ;

	/// Distribution to sample from
	Parameter< std::string > noiseDistribution ;

	Parameter< float > relNoiseAmplitude ;

	/// Update object.
	virtual void execute();
};

#endif // _TOFSIM_H_
