

// matte.cpp*
#include "measured.h"
#include "memory.h"
#include "singlebsdf.h"
#include "primitive.h"
#include "antiLambertian.h"
#include "texture.h"
#include "color.h"
#include "paramset.h"
#include "dynload.h"

using namespace lux;

// Matte Method Definitions
BSDF *Measured::GetBSDF(MemoryArena &arena, const SpectrumWavelengths &sw,
	const Intersection &isect, const DifferentialGeometry &dgs) const
{
	// Allocate _BSDF_
	// Evaluate textures for _Matte_ material and allocate BRDF
	// NOTE - lordcrc - changed clamping to 0..1 to avoid >1 reflection
	//SWCSpectrum r = Kd->Evaluate(sw, dgs).Clamp(0.f, 1.f);
	float color[4] = {0.5,0.5,0.5,0.5} ;
	SWCSpectrum r(color) ;
	BxDF *bxdf;
	bxdf = ARENA_ALLOC(arena, AntiLambertian)(r);

	SingleBSDF *bsdf = ARENA_ALLOC(arena, SingleBSDF)(dgs,
		isect.dg.nn, bxdf, isect.exterior, isect.interior);

	// Add ptr to CompositingParams structure
	bsdf->SetCompositingParams(&compParams);

	return bsdf;
}
Material* Measured::CreateMaterial(const Transform &xform,
		const ParamSet &mp) {
	return new Measured(mp);
}

static DynamicLoader::RegisterMaterial<Measured> r("measured");
